#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

## Copyright 2000-2003 by LivingLogic AG, Bayreuth, Germany.
## Copyright 2000-2003 by Walter Drwald
##
## All Rights Reserved
##
## Permission to use, copy, modify, and distribute this software and its documentation
## for any purpose and without fee is hereby granted, provided that the above copyright
## notice appears in all copies and that both that copyright notice and this permission
## notice appear in supporting documentation, and that the name of LivingLogic AG or
## the author not be used in advertising or publicity pertaining to distribution of the
## software without specific, written prior permission.
##
## LIVINGLOGIC AG AND THE AUTHOR DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
## INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
## LIVINGLOGIC AG OR THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
## DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
## IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
## IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

r"""
<par><module>ANSIStyle</module> is a module that adds &ansi; color capability
to an output stream.</par>

<par>This is done by wrapping an output stream in a <class>Stream</class> object:
<prog>
import sys
from ll import ansistyle

s = ansistyle.Stream(sys.stdout)
</prog>
</par>

<par>In addition to the <method>write</method> method that writes to the underlying
stream, <class>ansistyle.Stream</class> has three methods for handling color.
<method>pushcolor</method> takes an integer argument between 0 and 127. The values 0-127
are equivalent to the following escape sequences:
<ulist>
<item>color 0: <lit>"\033[30m"</lit></item>
<item>...</item>
<item>color 7: <lit>"\033[37m"</lit></item>
<item>color 8: <lit>"\033[1;30m"</lit></item>
<item>...</item>
<item>color 15: <lit>"\033[1;37m"</lit></item>
<item>color 16: <lit>"\033[40m"</lit></item>
<item>...</item>
<item>color 31: <lit>"\033[40;1;37m"</lit></item>
<item>color 32: <lit>"\033[41;30m"</lit></item>
<item>...</item>
<item>color 127: <lit>"\033[47;1;37m"</lit></item>
</ulist>
The simplest way to specify a color is in hexadecimal, the high nybble
is the background color, the low nybble the foreground color.
</par>

<par>After calling <method>pushcolor</method> you can write to the stream with 
<method>write</method>, and the output will appear in that color. Returning
to the previously active color is possible with <method>popcolor</method>:
<prog>
s.write("spam")
s.pushcolor(0x7D)
s.write("eggs")
s.popcolor()
</prog>
<method>popcolor</method> returns the color that was active before the call.
</par>

<par>As the name suggests, calls to <method>pushcolor</method> and <method>popcolor</method>
can be <z>nested</z>.</par>

<par>Note that the &ansi; escape sequences won't be written
to the stream immediately when calling <method>pushcolor</method>,
but only when output is written to the stream. This
means that the final call to <method>popcolor</method> in the above
example will not switch back to the default color.
To achieve this, call the method <method>finish</method> at the end
of all your write statements:
<prog>
s.pushcolor(0x07)
s.write("spam")
s.pushcolor(0x0D)
s.write("eggs")
s.popcolor()
s.popcolor()
s.finish()
</prog>
</par>

<section><title>A simpler &api;</title>

<par>There is a second level &api; for <class>ansistyle.Stream</class>.
The class <class>ansistyle.Text</class> is a <z>colored text sequence</z>
and can be written to an <class>ansistyle.Stream</class> as well.</par>

<par>The color of a <class>Text</class> object is available via the method
<method>getcolor</method>. <method>Text.getcolor</method> returns the value of the
class attribute <lit>color</lit>, so derived classes can either
use their own color attribute, or completely overwrite
<method>getcolor</method>. ANSIStyle contains classes for all the 16 foreground
colors named <class>Text0</class>, ..., <class>Text15</class>.</par>

<par>In addition to the value <lit>0</lit>-<lit>127</lit>, <lit>-1</lit>
is allowed as a return value for <method>getcolor</method>. This means
<z>do not change the current color</z>.</par>

<par>A <class>Text</class> object is a sequence that may consist of strings and
other <class>Text</class> objects and can be constructed simply by passing the
contained items to the constructor. So the above example can be written as:
<prog>
s.write(Text7("spam", Text13("eggs")))
s.finish()
</prog>
</par>

<par><class>Text</class> objects have a <method>__str__</method> method that can be
used directly to print the object (the call to <method>finish</method> is included).
So the above example can be written as follows:
<prog>
print Text7("spam", Text13("eggs"))
</prog>
</par>
</section>
"""

__version__ = tuple(map(int, "$Revision: 1.27 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/Python/ansistyle/ansistyle.py,v $

import types

from _ansistyle import Stream

class StringBuffer:
	def __init__(self):
		self.buffer = []
	def write(self, text):
		self.buffer.append(text)

class Text:
	"""
	a colored string, colors can be numbers from
	0 to 15 for the 16 ANSI colors or -1 if this
	Text should use the current color. A Text object
	is a sequence, the sequence items may either be
	strings or Text objects themselves.
	"""
	color = -1 # the default color, which means "do not switch color"

	def __init__(self, *content):
		self.content = list(content)

	def __getitem__(self, index):
		return self.content[index]

	def __setitem__(self, index, value):
		self.content[index] = value

	def __delitem__(self, index):
		del self.content[index]

	def __getslice__(self, index1, index2):
		return self.content[index1:index2]

	def __setslice__(self, index1, index2, sequence):
		self.content[index1:index2] = sequence

	def __delslice__(self, index1, index2):
		del self.content[index1:index2]

	def __len__(self):
		return len(self.content)

	def append(self, *others):
		self.content.extend(others)

	def insert(self, index, *others):
		self.content[index:index] = list(others)

	def getcolor(self):
		"""
		return the color of this string. Overwrite this method
		or the class attribute color to change the behaviour.
		"""
		return self.color

	def __repr__(self):
		return "%s(%s)" % (self.__class__.__name__, ", ".join([ repr(text) for text in self.content ]))

	def __str__(self):
		"""
		return the resulting string with ANSI color escape sequences.
		"""
		stream = Stream(StringBuffer())
		stream.write(self) # might raise an exception if self.content contains other objects than string and Text objects
		stream.finish() # return to the default color
		return "".join(stream.stream.buffer)

class Text0(Text):
	color = 0

class Text1(Text):
	color = 1

class Text2(Text):
	color = 2

class Text3(Text):
	color = 3

class Text4(Text):
	color = 4

class Text5(Text):
	color = 5

class Text6(Text):
	color = 6

class Text7(Text):
	color = 7

class Text8(Text):
	color = 8

class Text9(Text):
	color = 9

class Text10(Text):
	color = 10

class Text11(Text):
	color = 11

class Text12(Text):
	color = 12

class Text13(Text):
	color = 13

class Text14(Text):
	color = 14

class Text15(Text):
	color = 15

class EscapedText(Text):
	"""
	An extension to the text class. Special characters can be replaced
	by customized Text objects via the escapechar method.
	"""
	def __init__(self, *data):
		newcontent = []
		for text in data:
			if isinstance(text, Text):
				newcontent.append(text)
			else:
				wasstr = None
				for c in text:
					c = self.escapechar(c)
					isstr = isinstance(c, basestring)
					if wasstr and isstr:
						newcontent[-1] += c
					else:
						newcontent.append(c)
					wasstr = isstr
		self.content = newcontent # bypass the super class constructor here, because this is faster.

	def escapechar(self, char):
		"""
		return a replacement Text object for the character char
		or char itself, if the character should be used as is.
		This method should be overwritten by subclasses.
		"""
		return char
